#include "showplayer.h"


ShowPlayer::ShowPlayer(QWidget *parent) :
    QDialog(parent)
  , ui(new Ui::ShowPlayer)
{
    ui->setupUi(this);
    connect(ui->NewCue, SIGNAL(clicked()), this, SLOT(onAddTrack()));
    connect(ui->EditCue, SIGNAL(clicked()), ui->cueListWidget, SLOT(editCueTrack()));
    connect(ui->RemoveCue, SIGNAL(clicked()), ui->cueListWidget, SLOT(deleteCueTrack()));
    connect(ui->SaveCueList, SIGNAL(clicked()), this, SLOT(saveCueTrackList()));
    connect(ui->LoadCueList, SIGNAL(clicked()), this, SLOT(loadCueTrackList()));
    connect(ui->goButton, SIGNAL(clicked()), this, SLOT(go()));
    currentStatus = Status::Iddle;
    for(int i = 0; i < MAX_LAYERS; i++) {
       layersUsed[i] = -1;
       cueTrackWidgetPlaying[i] = nullptr;
    }
}

ShowPlayer::~ShowPlayer() {}

void ShowPlayer::onAddTrack() {
	ui->cueListWidget->createNewCueTrack();
}

void ShowPlayer::go()
{
    CueTrack *goCue = ui->cueListWidget->getSelectedTrack(true);
    if (!goCue)
        return;
    current[goCue->audioLayer] = goCue;
    playCueTrack(goCue);
    if (goCue->multi)
		go();
}

void ShowPlayer::goAction(int c)
{
    if (!current[c]->filePath.empty()) {
        emit uiLoadMedia(current[c]->audioLayer, current[c]->filePath.data());
        ui->cueListWidget->cueTrackAtEnd(c);
    }
    updateTrackStateInEngine(current[c]);
    emit uiPlaybackChanged(current[c]->audioLayer, current[c]->status);
    switch (current[c]->status) {
    case Status::PlayingOnce:
    case Status::PlayingLoop:
    case Status::PlayingFolder:
    case Status::PlayingFolderLoop:
    case Status::PlayingFolderRandom:
        current[c]->active = true;
        layersUsed[current[c]->audioLayer] = current[c]->userNumber;
        break;
    default:
        current[c]->active = false;
    }
    ui->cueListWidget->redrawCueTrackList();
}

void ShowPlayer::updateTrackStateInEngine(CueTrack *track) {
    emit uiSliderChangedFaded(track->audioLayer, Slider::Volume, track->volume * 655.35, track->fadeIn * 1000);
    emit uiSliderChanged(track->audioLayer, Slider::Pan, (track->pan + 1) * 128);
    emit uiSliderChanged(track->audioLayer, Slider::Pitch, track->pitch * 128);
    emit uiSliderChanged(track->audioLayer, Slider::Bus1, track->bus1 * 255 * 2.55);
    emit uiSliderChanged(track->audioLayer, Slider::Bus2, track->bus2 * 255 * 2.55);
}

void ShowPlayer::loadCueTrackList()
{
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("XML Files (*.xml)"));
	if (fileName.isEmpty())
		return;
     ui->cueListWidget->loadCueTrackList(fileName.toStdString());
}

void ShowPlayer::saveCueTrackList()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("XML Files (*.xml)"));
	if (fileName.isEmpty())
		return;
	ui->cueListWidget->saveCueTrackList(fileName.toStdString());
}

void ShowPlayer::cueTrackAtEnd(int layer)
{
    current[layer]->active = false;
    ui->cueListWidget->cueTrackAtEnd(layer);
    removeCueTrackWidget(layer);
}

void ShowPlayer::cueFinished(int c)
{
	if (current[c] && !current[c]->stopAtEnd)
        this->go();
}

CueTrackWidget *ShowPlayer::addCueTrackWidget(CueTrack* track) {
	QListWidgetItem* item = new QListWidgetItem(ui->activeCueList);
	CueTrackWidget* widget = new CueTrackWidget();
	widget->loadCueTrack(track);
	connect(widget, SIGNAL(goAction(int)), this, SLOT(goAction(int)));
    connect(widget, SIGNAL(cueFinished(int)), this, SLOT(cueFinished(int)));
    cueTrackWidgetPlaying[track->audioLayer] = widget;
    item->setSizeHint(widget->sizeHint());
	ui->activeCueList->setItemWidget(item, widget);
    return (widget);
}

void ShowPlayer::playCueTrack(CueTrack* track) {
    removeCueTrackWidget(track->audioLayer);
    CueTrackWidget *widget = addCueTrackWidget(track);
    widget->go();
}

void ShowPlayer::removeCueTrackWidget(int audioLayer) {
	CueTrackWidget* widgetToRemove = cueTrackWidgetPlaying[audioLayer];
    if (widgetToRemove != NULL && ui->activeCueList->count() > 0) {
		for (int i = 0; i < ui->activeCueList->count(); ++i) {
			QListWidgetItem* item = ui->activeCueList->item(i);
			QWidget* widget = ui->activeCueList->itemWidget(item);
			if (widget == widgetToRemove) {
				ui->activeCueList->removeItemWidget(item);
				delete item;
                delete widgetToRemove;
                break;
			}
		}
        cueTrackWidgetPlaying[audioLayer] = nullptr;
	}
}
